from vcCommand import *

app = getApplication()
teach = getApplication().TeachContext

def OnStart():
  routine = teach.ActiveScope
  
  processStatement = routine.addStatement(VC_STATEMENT_PROCESS, teach.ActiveStatementIndex + 1)
    
  name = processStatement.getProperty('Name')
  base = processStatement.getProperty('Base')
  tool = processStatement.getProperty('Tool')
  
  processStatement.createProperty(VC_VECTOR, 'ParentVector')
  processStatement.createProperty(VC_STRING, 'ParentName')
  processStatement.createProperty(VC_BUTTON, 'SetParentVector')  
  
  tolerance = processStatement.createProperty(VC_REAL, 'Tolerance')  
  tolerance.Quantity = 'Distance'
  tolerance.Value = 10.0   
  processStatement.createProperty(VC_STRING, 'ExcludeNames')
  
  robot = teach.ActiveRobot 
  attachStatementScriptName = 'AttachStatementScript_' + processStatement.Name
  attachStatementScript = robot.findBehaviour(attachStatementScriptName)
  if not attachStatementScript:
    attachStatementScript = robot.createBehaviour(VC_PYTHONPROCESSHANDLER, attachStatementScriptName)
  attachStatementScript.Script = '''
from vcPythonProcessHandler import *
import vcVector

app = getApplication()
snap = app.findCommand('interactiveSnap') 

def ParentPicked(arg):
  global parentVector, parentName
  target = snap.TargetNode.Component  
  parentVector.Value = target.WorldPositionMatrix.P
  parentName.Value = target.Name

def OnPickComponent(arg):
  snap.clear()   
  snap.OnTargetSet = ParentPicked    
  snap.execute()

def hideOthers(statement):
  name = statement.getProperty('Name')
  name.IsVisible = False
  base = statement.getProperty('Base')
  base.IsVisible = False
  tool = statement.getProperty('Tool')
  tool.IsVisible = False

def setEvents(statement):
  global parentVector, parentName
  
  parentVector = statement.getProperty('ParentVector')
  parentName = statement.getProperty('ParentName')
 
  global setParentVector, setChildVector
  setParentVector = statement.getProperty('SetParentVector')
  if setParentVector:
    if not setParentVector.OnChanged:
      setParentVector.OnChanged = OnPickComponent  
  
  global tolerance, excludeNames
  tolerance = statement.getProperty('Tolerance')
  excludeNames = statement.getProperty('ExcludeNames')
  
def OnStatementAdd(statement):
  # Hide others
  hideOthers(statement)
  
  # Set events
  setEvents(statement)

def OnStatementModified(statement):
  # Hide others
  hideOthers(statement)
  
  # Set events
  setEvents(statement)

def getCollisionDetector():  
  detector = app.findLayoutItem('AttachStatement_v2_CollisionDetector')
  if not detector:
    detector = app.createLayoutItem(VC_LAYOUTITEM_IT_COLLISIONDETECTOR)
    detector.Name = 'AttachStatement_v2_CollisionDetector'
  return detector
  
def OnStatementExecute(executor, statement):  
  # Hide others
  hideOthers(statement)
  
  # Set events
  setEvents(statement)
    
  excludes = excludeNames.Value.split(',')
  excludes = [exclude.strip() for exclude in excludes]
 
  for part in app.Components:    
    if part.Name == parentName.Value and (part.WorldPositionMatrix.P - parentVector.Value).length() < 1e-3:             
      detector = getCollisionDetector()
      detector.NodeListA = [(part, VC_NODELIST_INCLUDE, VC_NODELIST_TREE )]
      detector.NodeListB = [(app.Simulation.World, VC_NODELIST_INCLUDE, VC_NODELIST_TREE)]
      detector.NodeListB.append((part, VC_NODELIST_EXCLUDE, VC_NODELIST_TREE))
      hit = detector.testAllCollisions(tolerance.Value)
      
      if hit:
        for hitIndex in range(detector.HitCount):          
          child = detector.getHitNodeB(hitIndex).Component
          
          # Pass parent and check excludes
          if child != part.Parent and child.Name not in excludes:            
            matrix = part.InverseWorldPositionMatrix * child.WorldPositionMatrix
            part.attach(child)
            child.PositionMatrix = matrix
      return      
'''

  processStatement.Process = attachStatementScript
  app.SelectionManager.setSelection(processStatement)
